{% extends "appbuilder/general/model/show.html" %}

{% block content %}

    {{ lib.panel_begin("What is this") }}

    This is the detail of your contact
    {{ lib.panel_end() }}

    {{ super() }}

{% endblock content %}


{% block show_form %}
    Text before the show section
    <div>
    {{ extra_arg_obj1 }}
    </div>

    {{ super() }}
{% endblock show_form %}
