{% extends 'appbuilder/init.html' %}
{% import 'appbuilder/baselib.html' as baselib %}

{% block body %}
{% include 'appbuilder/general/confirm.html' %}
{% include 'appbuilder/general/alert.html' %}

{% block navbar %}
<header class="top" role="header">
    {% include 'appbuilder/navbar.html' %}
</header>
{% endblock %}

<div class="container">
    <div class="row">
        {% block messages %}
        {% include 'appbuilder/flash.html' %}
        {% endblock %}
        {% block content %}
        {% endblock %}
    </div>
</div>

{% block footer %}
<footer>
    <div class="img-rounded nav-fixed-bottom">
        <div class="container">
            {% include 'appbuilder/footer.html' %}
        </div>
    </div>
</footer>
{% endblock %}
{% endblock %}

{% block tail_js %}
{{ super() }}
<script src="/static/dist/main.js"></script>
{% endblock %}
